'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _supertest = require('supertest');

var _supertest2 = _interopRequireDefault(_supertest);

var _server = require('../../sample/basic/server');

var _server2 = _interopRequireDefault(_server);

var _index = require('../../index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var expect = _chai2.default.expect;


describe('Basic', function () {
  describe('Express API Versioning', function (done) {
    it('should throw an exception when api path is not explicitly set', function () {
      expect(function () {
        return (0, _index2.default)({ instance: function instance(f) {
            return f;
          } });
      }).to.throw('You must explicitly specify a path to where the APIs reside');
    });

    it('should throw an exception when an instance of express is not set', function () {
      expect(function () {
        return (0, _index2.default)({ apiPath: '../index.js' });
      }).to.throw('You must explicitly set an instance of express');
    });

    it('should throw an exception when api path cannot be found', function () {
      (0, _supertest2.default)(_server2.default).get('/api/v4').end(function (error, response) {
        expect(response.statusCode.to.equal(500));
        if (error) return done(error);
        done();
      });
    });

    it('should throw an exception when an instance of express is not a function', function () {
      expect(function () {
        return (0, _index2.default)({
          apiPath: '../index.js',
          instance: 'An instance'
        });
      }).to.throw('An instance of express must be a function but got type string');
    });

    it('should throw an error if the endpoint does not match the test condition', function () {
      (0, _supertest2.default)(_server2.default).get('/api/unknown').end(function (err, response) {
        expect(response.statusCode).to.equal(500);
        if (err) return done(err);
        done();
      });
    });

    it('should return 200 Ok when a route is found', function () {
      (0, _supertest2.default)(_server2.default).get('/api/v1').end(function (err, res) {
        expect(res.statusCode).to.equal(200);
        expect(res.body.message).to.be.a('string');
        if (err) return done(err);
        done();
      });
    });
  });
});